/****************************/
/*		MATH OPTIMIZATIONS	*/
/* (c)2004 Pangea Software  */
/* By Brian Greenstone      */
/****************************/


/****************************/
/*    PROTOTYPES            */
/****************************/

static void DrawSpinningCube(void);


/**************************/
/*    EXTERNS             */
/**************************/

extern	Boolean						gPlayInWindow;
extern	short						gDesiredColorDepth, gDesiredRezH, gDesiredRezV;
extern	CGDirectDisplayID			gCGDisplayID;
extern	int							gFrameCounter;


/****************************/
/*    VARIABLES             */
/****************************/

CFBundleRef 		gBundle = nil;
IBNibRef 			gNibs = nil;

		/* GEOMETRY DATA FOR CUBE */
		
static	OGLPoint3D	gCubePoints[8] =
{
	-1, -1, 1,					// left/bottom/front
	-1,-1,-1,					// left/bottom/rear
	1,-1,-1,					// right/bottom/rear
	1,-1,1,						// right/bottom/front
	-1, 1, 1,					// left/top/front
	-1,1,-1,					// left/top/rear
	1,1,-1,						// right/top/rear
	1,1,1,						// right/top/front	
};

static	OGLColorRGBA	gVertexColors[8] =
{
	0,0,0,1,					
	1,0,0,1,					
	0,1,0,1,				
	0,0,1,1,				
	1,1,1,1,				
	1,1,0,1,			
	0,1,1,1,				
	1,0,1,1,			
};

static	GLint	gCubeQuads[6*4] =
{
	0,1,2,3,					// bottom face
	7,6,5,4,					// top face
	4,5,1,0,					// left face
	7,3,2,6,					// right face
	7,4,0,3,					// front face
	5,6,2,1,					// rear face
};



/***************** DRAW SCENE GEOMETRY *******************/
//
// Called from DrawOpenGLScene() when it's time to submit our
// geometry data to OpenGL.
//

void DrawSceneGeometry(void)
{
static	OGLPoint3D	cameraCoord = {0,0,20};
static	OGLPoint3D	lookAtPt	= {0,0,0};
static	OGLVector3D	upVec		= {0,1,0};

			/* DRAW A FRAME COUNTER IN THE UPPER-LEFT CORNER */
			
	OGL_DrawString("\pframe #:", 20, 20);
	OGL_DrawInt(gFrameCounter, 70, 20);
	
			
			/* DRAW MESSAGE IN OPPOSITE CORNER */
			
	OGL_DrawString("\pClick the mouse to Exit", 500, 440);
		
	

			/* SET CAMERA & STATE */
				
	OGL_SetCamera(&cameraCoord, &lookAtPt, &upVec);
	OGL_SetColor4f(1,1,1,1);	
	OGL_DisableLighting();
	OGL_DisableTexture2D();
	OGL_DisableBlend();
	
			/* DRAW CUBE */
			
	DrawSpinningCube();
}



/***************** DRAW SPINNING CUBE *******************/

static void DrawSpinningCube(void)
{
static	float	ry = 0, rx = 0;

	
			/* SCALE AND ROTATE */
			
	glScalef(3,3,3);
	glRotatef(rx += .11,1,0,0);
	glRotatef(ry += .10,0,1,0);


			/* INIT THE VERTEX ARRAYS */
						
	glEnableClientState(GL_VERTEX_ARRAY);			// point to points
	glVertexPointer(3, GL_FLOAT, 0, gCubePoints);	

	glColorPointer(4, GL_FLOAT, 0, gVertexColors);
	glEnableClientState(GL_COLOR_ARRAY);			// point to vertex colors

	glDisableClientState(GL_TEXTURE_COORD_ARRAY);	// no uv's
	glDisableClientState (GL_NORMAL_ARRAY);			// no normals

		
			/* SUBMIT THE VERTEX ARRAYS */

	glDrawElements(GL_QUADS, 4*6, GL_UNSIGNED_INT, gCubeQuads);
		
}

#pragma mark -


/******************** GET GAME NIB *********************/
//
// Our dialog resources are all stored in the Game.nib file.
// This code creates a reference to that .nib file so that we can
// easily get to the resources whenever we need to.
//

static void GetGameNib(void)
{
OSStatus	iErr;
			
	gBundle = CFBundleGetMainBundle();
	if (gBundle == nil)
		DoFatalAlert("\pCannot locate Main Bundle!");
		
	iErr = CreateNibReferenceWithCFBundle(gBundle, CFSTR("Game"), &gNibs);			// note:  we don't need the ".nib" extension in the filename
	if (iErr != noErr)
		DoFatalAlert("\pCreateNibReferenceWithCFBundle failed!");
}


/************************ MAIN **************************/

int main(void)
{
			/* INITIALIZE THINGS */
			
	GetGameNib();
	
	
			/* ASK USER FOR VIDEO SETTINGS */
			
	DoScreenModeDialog();

	
	if (gPlayInWindow)							// play in window or full screen?
		CreateGameWindow();		
	else	
		SwitchDisplayMode(gDesiredRezH, gDesiredRezV, gDesiredColorDepth);	

	CalcDisplayVRAM();							// calc amount of VRAM on this display
	

			/* GET OPENGL GOING */
				
	OGL_CreateDrawContext();					// create the draw context
	OGL_InitFont();								// init an AGL font to use
	
	
			/* DRAW STUFF */
			
	while (!Button())							// loop until user presses mouse button
	{
		OGL_DrawScene();	
	}
	
}










